@echo off

if "%1" == "" goto NoArgs

REM ----- Setup compilation and release process -----

setlocal

set psirelease=daisy
set name=Psi-0.12-Daisy for Windows
set project=psi.pro
set nsiscript=

set year=%date:~0,4%
set month=%date:~5,2%
set day=%date:~8,2%

set debug=

set rootdir=%cd%
set binoutdir=psiwin
set logfilename=%rootdir%\makewin.log
set nsislogfilename=%rootdir%\makewin_nsis.log

set system=%windir%\system
if "%OS%" == "Windows_NT" set system=%system%32

set delfile=del /f /q
set deldir=rmdir /s /q
set copyfile=copy
set copydir=xcopy /c /e /i /q

call %1 > nul
shift

REM ----- Compile and link Psi -----

echo This will compile, link and release %name%.
set /a step=1
echo STEP %step%: Create Makefile for %name%
title Creating Makefile for %name%...
qmake %project% >> %logfilename%
if errorlevel 1 goto NoQMake
set /a step=((%step% + 1))
echo STEP %step%: Compile %name%
title Compiling %name%...
if exist src\_obj\applicationinfo.o (
	%delfile% src\_obj\applicationinfo.o > nul
)
%MAKE% >> "%logfilename%"
if errorlevel 1 goto CompileFailed

REM ----- Decide whether this is debug-only build -----

if "%1" == "debug" goto Release
if not "%debug%" == "" goto Release

REM ----- Build plugins -----

goto skipplugins
set /a step=((%step% + 1))
echo STEP %step%: Build plugins
title Building plugins...
for %%d in (src\plugins\generic\chess src\plugins\generic\consoledump src\plugins\generic\echo src\plugins\generic\noughtsandcrosses src\plugins\generic\null src\plugins\generic\python src\plugins\generic\urlwatcher src\plugins\windows\snarl) do (
	echo      - Creating Makefile for '%%d' plugin
	pushd %%d
	qmake >> %logfilename%
	echo      - Compiling '%%d' plugin
	%MAKE% >> "%logfilename%"
	if errorlevel 1 goto CompileFailed
	popd
)
:skipplugins

REM ----- Copy output to single directory -----

:Release
set /a step=((%step% + 1))
echo STEP %step%: Prepare for release of %name%
title Preparing for release of %name%...
pushd win32
set WINDOWSDIR=%windir%
set TARGET_DIR=..\%binoutdir%
echo         Preliminary cleanup...
if exist %binoutdir% (
	%deldir% %binoutdir% > nul
)
echo         Main binaries (including aspell dictionaries)...
%MAKE% >> "%logfilename%"
if errorlevel 1 goto CompileFailed
popd
if exist src\plugins\windows\snarl\release (
	if exist src\plugins\windows\snarl\release\*.dll (
		echo         Psi Snarl support plugin...
		if not exist %binoutdir%\plugins (
			mkdir %binoutdir%\plugins
		)
		%copyfile% src\plugins\windows\snarl\release\*.dll %binoutdir%\plugins > nul
	)
)
if exist %QTDIR%\plugins\imageformats (
	if exist %QTDIR%\plugins\imageformats\q*.dll (
		echo         Qt image support plugins...
		if not exist %binoutdir%\imageformats (
			mkdir %binoutdir%\imageformats
		)
		%copyfile% %QTDIR%\plugins\imageformats\q*.dll %binoutdir%\imageformats > nul
		%delfile% %binoutdir%\imageformats\q*d*.dll > nul
	)
)
if exist lang (
	if exist lang\*.qm (		
		echo         Existing translations...
		%copyfile% lang\*.qm %binoutdir% > nul
	)
)

REM ----- Decide whether this is debug-only build -----

if "%1" == "debug" goto Finished
if not "%debug%" == "" goto Finished

REM ----- Create installers -----

if "%nsiscript%" == "" goto NoInstaller
if not exist %nsiscript% goto NoInstaller
set /a step=((%step% + 1))
echo STEP %step%: Prepare installer for %name%
title Preparing installer for %name%...
if not "%1" == "debug" (
	makensis /V4 /O%nsislogfilename% /D%COMPILER% /DYEAR=%year% /DMONTH=%month% /DDAY=%day% %nsiscript%
)
type %nsislogfilename% >> %logfilename%
if errorlevel 1 goto NSISFailed
set /a step=((%step% + 1))
echo STEP %step%: Prepare full installer for %name%
title Preparing full installer for %name%...
if not "%1" == "debug" (
	makensis /V4 /O%nsislogfilename% /D%COMPILER% /DYEAR=%year% /DMONTH=%month% /DDAY=%day% /DFULL %nsiscript%
)
type %nsislogfilename% >> %logfilename%
if errorlevel 1 goto NSISFailed
:NoInstaller

REM ----- Archive binaries and sources -----

set /a step=((%step% + 1))
echo STEP %step%: Package %name%
call %rootdir%\makewin_package.bat %psirelease% %binoutdir%
if errorlevel 1 goto ArchiveFailed
:DontArchive

REM ----- Process succeeded -----

:Finished
echo Seems that %name% is ready for distibution.
title Done.
goto End


REM ----- Error handling -----

:NoArgs
echo No arguments at command line.
goto End

:NoQMake
echo Failed to run 'qmake'. Are you sure you have Qt set up?
goto End

:CopyFailed
echo Copy failed, stopping.
goto End

:CompileFailed
echo Compile failed, stopping. See 'makewin.log' file for details.
goto End

:NSISFailed
echo Installer generation failed, stopping. Maybe some files are missing?
goto End

:ArchiveFailed
echo Archive generation failed, stopping.
goto End


REM ----- Cleanup -----

:End
endlocal
